package com.wxw.nfcdemo;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.util.Locale;

import com.umeng.analytics.MobclickAgent;
import com.umeng.analytics.MobclickAgent.EScenarioType;

import android.app.Activity;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.nfc.FormatException;
import android.nfc.NdefMessage;
import android.nfc.NdefRecord;
import android.nfc.NfcAdapter;
import android.nfc.Tag;
import android.nfc.tech.MifareClassic;
import android.nfc.tech.Ndef;
import android.nfc.tech.NdefFormatable;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.AdapterView.OnItemSelectedListener;

public class NfcMifareActivity extends Activity {
    private Context mContext;
    private final String mPageName = "NfcMifareActivity";
    private static final String TAG = NfcMifareActivity.class.getSimpleName();
    private static final String LINE_SEPARATOR = System.getProperty("line.separator", "\n");

    private boolean mDebug = true;
    boolean auth;

    private String mCharset = "UTF-8";

    private NfcAdapter mAdapter;
    private PendingIntent mPendingIntent;
    private IntentFilter[] mIntentFilters;
    private String[][] mTechList;
    private EditText et;

    private EditText mWriteBlockContent;


    private Button btn_w;

    private TextView tx;

    int wBlock;

    int checkBlock = 0;

    String retData;

    private Spinner spinner;
	ArrayAdapter<CharSequence> adapter1;
	ArrayAdapter<CharSequence> adapter2;
	ArrayAdapter<CharSequence> adapter3;

	private MifareClassic mfc;

	private Ndef ndef;

	private NdefFormatable ndefFormat;

	private NdefMessage NDEFMsg2Write = null;

	private LinearLayout mfc1;

	private LinearLayout mfc2;



    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.nfc_write);

		mContext=this;
		MobclickAgent.setDebugMode(true);
		MobclickAgent.setScenarioType(mContext, EScenarioType.E_UM_NORMAL);

        mAdapter = NfcAdapter.getDefaultAdapter(this);
        if (mAdapter == null) {
            this.finish();
            return;
        }

        mPendingIntent = PendingIntent.getActivity(this, 0,
        		new Intent(this, getClass()).addFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP), 0);
        IntentFilter techfilter = new IntentFilter(NfcAdapter.ACTION_TECH_DISCOVERED);
//        IntentFilter ndeffilter = new IntentFilter(NfcAdapter.ACTION_NDEF_DISCOVERED);
//        IntentFilter tagfilter = new IntentFilter(NfcAdapter.ACTION_TAG_DISCOVERED);
        try {
//        	techfilter.addDataType("*/*");
//        	ndeffilter.addDataType("*/*");
        } catch (Exception e) {
            e.printStackTrace();
        }
        mIntentFilters = new IntentFilter[]{techfilter};
        mTechList = new String[][]{new String[]{MifareClassic.class.getName()},
        		new String[]{Ndef.class.getName()},
        		new String[]{NdefFormatable.class.getName()}};


        initViews();
        onNewIntent(getIntent());
    }

    @Override
    protected void onResume() {
        super.onResume();
        mAdapter.enableForegroundDispatch(this, mPendingIntent, mIntentFilters, mTechList);
        MobclickAgent.onPageStart(mPageName);
        MobclickAgent.onResume(mContext);
    }

    @Override
    protected void onPause() {
        super.onPause();
        mAdapter.disableForegroundDispatch(this);
        MobclickAgent.onPageEnd(mPageName);
        MobclickAgent.onPause(mContext);
    }

    @Override
    protected void onNewIntent(Intent intent) {
        if (!NfcAdapter.ACTION_TECH_DISCOVERED.equals(intent.getAction())) {
            return;
        }
        resolveIntent(intent);
    }



    private void resolveIntent(Intent intent) {
        debug("resolveIntent...", false);

        Tag  detectTag = intent.getParcelableExtra(NfcAdapter.EXTRA_TAG);

        ndefFormat = NdefFormatable.get(detectTag);
        ndef = Ndef.get(detectTag);
        mfc = MifareClassic.get(detectTag);
        if(supportedTechs(detectTag.getTechList())){
        	mfc1.setVisibility(View.GONE);
        	mfc2.setVisibility(View.GONE);
        	tx.setText(getString(R.string.setText3));
        	return;
        }
        else if (mfc == null) {
            debug(getString(R.string.toast_not_mifare_classic), true);
            return;
        }
        else if (!(mfc == null)){
			try {

				mfc.connect();
				int sectorCount = mfc.getSectorCount();// ��ȡTAG�а��������
				mfc1.setVisibility(View.VISIBLE);
				mfc2.setVisibility(View.VISIBLE);
				if(sectorCount == 16){
					spinner.setAdapter(adapter2);
					tx.setText(getString(R.string.setText1));
					spinner.setSelection(checkBlock, true);}
				else if(sectorCount == 32){
					spinner.setAdapter(adapter3);
				    tx.setText(getString(R.string.setText2));
				    spinner.setSelection(checkBlock, true);}
				else {
					spinner.setAdapter(adapter2);
					tx.setText(getString(R.string.setText));
					spinner.setSelection(checkBlock, true);}
			}catch (Exception e) {
				e.printStackTrace();
			}finally {
				try {
					mfc.close();
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}


        }else{
        	debug(getString(R.string.not_support),true);
        	return;
        }

    }

    boolean writeNdefMessageToTag(NdefMessage message) {

        int size = message.toByteArray().length;


           if (ndef != null) {
            	ndefConnect(ndef);

                if(!ndef.isConnected()){
	    			debug(getString(R.string.find_tag), true);
		            return true;
		            }
              try {
                if (!ndef.isWritable()) {
                    debug(getString(R.string.read_only),true);
                    return true;
                }
                if (ndef.getMaxSize() < size) {
                	debug(getString(R.string.size_check),true);

                    return true;
                }

                ndef.writeNdefMessage(message);
                ndef.close();

                debug(getString(R.string.toast_write_success),true);

                return true;
            }catch(Exception e){return true;}
           } else {

                if (ndefFormat != null) {


							ndefFormatConnect(ndefFormat);

                        if(!ndefFormat.isConnected()){
        	    			debug(getString(R.string.find_tag), true);
        		            return true;
        		            }
                        try {
                        try {
							ndefFormat.format(message);
						} catch (FormatException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
                        ndefFormat.close();
                        debug(getString(R.string.toast_write_success),true);

                        return true;
                    } catch (IOException e) {

                    	debug(getString(R.string.toast_write_fail),true);
                        Toast.makeText(this, "Failed to format tag", Toast.LENGTH_SHORT).show();
                        return true;
                    }
                } else {

                    return false;
                }
            }

    }

    public static boolean supportedTechs(String[] techs) {


        boolean ultralight = false;
        boolean nfcA = false;
        boolean ndef = false;

        for (String tech : techs) {
            if (tech.equals("android.nfc.tech.MifareUltralight")) {
                ultralight = true;

            } else if (tech.equals("android.nfc.tech.NfcA")) {
                nfcA = true;

            } else if (tech.equals("android.nfc.tech.Ndef")
                    || tech.equals("android.nfc.tech.NdefFormatable")) {
                ndef = true;

            } else if (tech.equals("android.nfc.tech.MifareClassic")) {

            }
        }
        if (ultralight && nfcA && ndef) {
            return true;
        } else {
            return false;
        }
    }


    public static NdefMessage getNdefMsg_from_RTD_TEXT(String text, boolean encodeInUtf8) {

        Locale locale = new Locale("en", "US"); // a new Locale is created with
                                                // US English.
        byte[] langBytes = locale.getLanguage().getBytes(Charset.forName("US-ASCII"));
        // boolean encodeInUtf8 = false;
        Charset utfEncoding = encodeInUtf8 ? Charset.forName("UTF-8") : Charset.forName("UTF-16");
        int utfBit = encodeInUtf8 ? 0 : (1 << 7);
        char status = (char)(utfBit + langBytes.length);
        // String text = "This is an RTD_TEXT exp";
        byte[] textBytes = text.getBytes(utfEncoding);
        byte[] data = new byte[1 + langBytes.length + textBytes.length];
        data[0] = (byte)status;
        System.arraycopy(langBytes, 0, data, 1, langBytes.length);
        System.arraycopy(textBytes, 0, data, 1 + langBytes.length, textBytes.length);
        NdefRecord textRecord = new NdefRecord(NdefRecord.TNF_WELL_KNOWN, NdefRecord.RTD_TEXT,
                new byte[0], data);


            return new NdefMessage(new NdefRecord[] {
                textRecord
            });

    }

    private byte[] readBlock(MifareClassic mfc, int blockIndex) throws IOException {
        return mfc.readBlock(wBlock);
    }

    private void writeBlock(MifareClassic mfc, int blockIndex, byte[] data) throws IOException {
        mfc.writeBlock(blockIndex, data);
    }

    private String convertBytes2String(byte[] data) throws UnsupportedEncodingException {
        String ret;

            int pos = data.length;
            for (int i = data.length - 1; i >= 0; i--) {
                if (data[i] != 0) {
                    break;
                }
                pos = i;
            }
            ret = new String(data, 0, pos, mCharset);

        return ret;
    }

    private byte[] convertString2Bytes(String content) throws UnsupportedEncodingException {
        byte[] ret = new byte[16];
        byte[] buf = content.getBytes(mCharset);
        int retLen = ret.length;
        int bufLen = buf.length;
        boolean b = retLen > bufLen;

        for (int i = 0; i < retLen; i++) {
            if (b && i >= bufLen) {
                ret[i] = 0;
                continue;
            }
            ret[i] = buf[i];
        }
        return ret;
    }

    private String getText(EditText et) {
        if (TextUtils.isEmpty(et.getText())) {
            return null;
        }
        return et.getText().toString().trim();
    }



    private boolean validateBlockIndex(MifareClassic mfc, int blockIndex) {
        if (blockIndex >= mfc.getBlockCount()) {
            return false;
        }
        return true;
    }


    public void clearText(View view) {
        switch (view.getId()) {
            case R.id.btn_write_block_content:
                mWriteBlockContent.setText(null);
                break;
        }
    }

    private void initViews() {


        mWriteBlockContent = (EditText) findViewById(R.id.et_write_block_content);

        btn_w = (Button) findViewById(R.id.btn_w);

        spinner = (Spinner) findViewById(R.id.spn);

        tx= (TextView) findViewById(R.id.text);

        et= (EditText) findViewById(R.id.editText);

        mfc1 = (LinearLayout) findViewById(R.id.mfc_layout);
        mfc2 = (LinearLayout) findViewById(R.id.mfc_layout1);

		adapter1 = ArrayAdapter.createFromResource(this, R.array.MF_default, android.R.layout.simple_spinner_item);
		adapter2 = ArrayAdapter.createFromResource(this, R.array.MF1_card, android.R.layout.simple_spinner_item);
		adapter3 = ArrayAdapter.createFromResource(this, R.array.MF2_card, android.R.layout.simple_spinner_item);

		adapter1.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		adapter2.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		adapter3.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		spinner.setAdapter(adapter1);

		spinner.setOnItemSelectedListener(new MyOnItemSelectedListener());


        btn_w.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View arg0) {
				String content = getText(mWriteBlockContent);

				if(ndef==null && ndefFormat==null && mfc ==null){
					debug(getString(R.string.find_mfc),true);
					return;
				}
				if(content==null){
					debug(getString(R.string.write_content), true);
	    			return;
				}

	        	if((!(ndef==null) || !(ndefFormat ==null)) && mfc==null){
	        		NDEFMsg2Write = getNdefMsg_from_RTD_TEXT(content,true);
	        		writeNdefMessageToTag(NDEFMsg2Write);
	        	}else {
    			mfcConnect(mfc);
				if(!mfc.isConnected()){
	    	    			debug(getString(R.string.find_tag), true);
	    		            return;
	    		            }
				if (!validateBlockIndex(mfc, wBlock)) {
	                debug(getString(R.string.err_block_index_out_of_bound), true);
	                return;
	            }
						try {
							auth = mfc.authenticateSectorWithKeyA(mfc.blockToSector(wBlock), MifareClassic.KEY_DEFAULT);
						} catch (IOException e1) {
							// TODO Auto-generated catch block
							e1.printStackTrace();

						}
			            if (auth) {

				    		try {
								writeBlock(mfc, wBlock, convertString2Bytes(content));
								byte[] data = readBlock(mfc,wBlock);
			                    retData = convertBytes2String(data);


							} catch (UnsupportedEncodingException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							} catch (IOException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}

				    		StringBuffer sb = new StringBuffer();
				    		sb.append(String.format(getString(R.string.block_index_dynamic), wBlock));
			                sb.append(LINE_SEPARATOR);
			                sb.append(LINE_SEPARATOR);
			                sb.append(retData);
			                sb.append(LINE_SEPARATOR);
			                et.setText(sb.toString());
				    		debug(getString(R.string.toast_write_success), true);
			            } else{
			            	debug(getString(R.string.auth_failed), true);

			            }

			    		try{
			    			mfc.close();
			    		}catch(Exception e){
			    			e.printStackTrace();
			    		}
			    	}

			}

			});

    }

    public void mfcConnect(MifareClassic mfc){
    	try {
			mfc.connect();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }

    public void ndefConnect(Ndef ndef){
    	try {
			ndef.connect();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }

    public void ndefFormatConnect(NdefFormatable ndefFormat) {
    	try {
			ndefFormat.connect();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }

    class MyOnItemSelectedListener implements OnItemSelectedListener
	{

		@Override
		public void onItemSelected(AdapterView<?> parent, View view, int position, long id)
		{
			String str = parent.getItemAtPosition(position).toString();

			str = str.trim();
			checkBlock = position;

			String str2 = "";
			if(str != null && !"".equals(str)){
				for(int i=0;i<str.length();i++){
				if(str.charAt(i)>=48 && str.charAt(i)<=57){
				str2+=str.charAt(i);
				}
			}
				wBlock = Integer.parseInt(str2);

		}

		}

		@Override
		public void onNothingSelected(AdapterView<?> parent)
		{

		}

	}

    private void debug(String info, boolean toast) {
        if (mDebug) {
            Log.i(TAG, info);
            if (toast) {
                showToast(info);
            }
        }
    }

    private void showToast(String toast) {
        Toast.makeText(this, toast, Toast.LENGTH_SHORT).show();
    }
}
