package com.wxw.xnfc.card.pboc;

import java.util.ArrayList;

import android.content.res.Resources;

import com.wxw.nfcdemo.R;
import com.wxw.xnfc.Util;
import com.wxw.nfctech.Iso7816;

final class WuhanTong extends PbocCard {
	private final static int SFI_INFO = 5;
	private final static int SFI_SERL = 10;
	
	private final static byte[] DFN_SRV = { (byte) 0x41, (byte) 0x50,
			(byte) 0x31, (byte) 0x2E, (byte) 0x57, (byte) 0x48, (byte) 0x43,
			(byte) 0x54, (byte) 0x43, };

	private WuhanTong(Iso7816.Tag tag, Resources res) {
		super(tag);
		name = res.getString(R.string.name_wht);
	}

	@SuppressWarnings("unchecked")
	final static WuhanTong load(Iso7816.Tag tag, Resources res) {
		
		/*--------------------------------------------------------------*/
		// select PSF (1PAY.SYS.DDF01)
		/*--------------------------------------------------------------*/
		if (tag.selectByName(DFN_PSE).isOkey()) {

			Iso7816.Response SERL, INFO, CASH;

			/*--------------------------------------------------------------*/
			// read card info file, binary (5, 10)
			/*--------------------------------------------------------------*/
			if (!(SERL = tag.readBinary(SFI_SERL)).isOkey())
				return null;
			
			if (!(INFO = tag.readBinary(SFI_INFO)).isOkey())
				return null;

			CASH = tag.getBalance(true);
			
			/*--------------------------------------------------------------*/
			// select Main Application
			/*--------------------------------------------------------------*/
			if (tag.selectByName(DFN_SRV).isOkey()) {
				
				/*--------------------------------------------------------------*/
				// read balance
				/*--------------------------------------------------------------*/
				if (!CASH.isOkey())
					CASH = tag.getBalance(true);
								
				/*--------------------------------------------------------------*/
				// read log file, record (24)
				/*--------------------------------------------------------------*/
				ArrayList<byte[]> LOG = readLog(tag, SFI_LOG);

				/*--------------------------------------------------------------*/
				// build result string
				/*--------------------------------------------------------------*/
				final WuhanTong ret = new WuhanTong(tag, res);
				ret.parseBalance(CASH);
				ret.parseInfo(SERL, INFO);
				ret.parseLog(LOG);

				return ret;
			}
		}

		return null;
	}
	
	private void parseInfo(Iso7816.Response sn, Iso7816.Response info) {
		if (sn.size() < 27 || info.size() < 27) {
			serl = version = date = count = null;
			return;
		}

		final byte[] d = info.getBytes();
		serl = Util.toHexString(sn.getBytes(), 0, 5);
		version = String.format("%02d", d[24]);
		date = String.format("%02X%02X.%02X.%02X - %02X%02X.%02X.%02X", d[20],
				d[21], d[22], d[23], d[16], d[17], d[18], d[19]);
		count = null;
	}
}
