package com.example.scan.setting.test;

import java.util.Map;
import java.util.Random;

import com.nlscan.android.scan.ScanManager;
import com.nlscan.android.scan.ScanSettings;

import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Color;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RadioGroup;
import android.widget.TextView;
import android.widget.Toast;


public class MainActivity extends Activity {

	private TextView tv_broadcast_result;
	private EditText et_emulator_result;
	private RadioGroup radio_group;
	private Button btn_reset;
	Context mContext;

	private ScanManager mScanMgr;
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		mScanMgr=ScanManager.getInstance();
		mContext = getApplicationContext();
		initView();
	}

	private void initView()
	{
		tv_broadcast_result=(TextView)findViewById(R.id.tv_broadcast_result);
		et_emulator_result=(EditText)findViewById(R.id.et_emulator_result);
		radio_group=(RadioGroup)findViewById(R.id.radio_group);
		btn_reset=(Button)findViewById(R.id.btn_reset);
		btn_reset.setOnClickListener(new View.OnClickListener() {

			@Override
			public void onClick(View view) {
				et_emulator_result.setText("");
				tv_broadcast_result.setText("广播输出的结果显示区");
				tv_broadcast_result.setTextColor(getResources().getColor(android.R.color.black));
			}
		});

		//可选的输出模式设置
		radio_group.setOnCheckedChangeListener(new RadioGroup.OnCheckedChangeListener() {

			@Override
			public void onCheckedChanged(RadioGroup group, int checkedId) {

				int outputMode=-1;

				switch (checkedId) {

				//广播方式输出
				case R.id.radio_broadcast:
					outputMode=ScanSettings.Global.VALUE_OUT_PUT_MODE_BROADCAST;
					break;

				//模拟键方式输出
				case R.id.radio_emulate_key:
					outputMode=ScanSettings.Global.VALUE_OUT_PUT_MODE_EMULATE_KEY;
					break;

				//直接填充
				case R.id.radio_fill:
					outputMode=ScanSettings.Global.VALUE_OUT_PUT_MODE_FILLING;
					break;
				}

			    if(outputMode!=-1)
			        mScanMgr.setOutpuMode(outputMode);
			}
		});

		//获取所有的通用设置
		Map<String, String> settings=mScanMgr.getScanSettings();

		if (settings != null)
    	{
    		String sOutputMode=settings.get(ScanSettings.Global.OUT_PUT_MODE); //获取当前输出方式
    		int iOutputMode=Integer.parseInt(sOutputMode);

    		if(iOutputMode == ScanSettings.Global.VALUE_OUT_PUT_MODE_BROADCAST)
    			radio_group.check(R.id.radio_broadcast);
    		if(iOutputMode == ScanSettings.Global.VALUE_OUT_PUT_MODE_EMULATE_KEY)
    			radio_group.check(R.id.radio_emulate_key);
    		if(iOutputMode == ScanSettings.Global.VALUE_OUT_PUT_MODE_FILLING)
    			radio_group.check(R.id.radio_fill);
		}
        else
        {
            Toast.makeText(mContext, "获取识读头配置失败", Toast.LENGTH_SHORT).show();
        }

	}

	private void registerReceiver()
	{
		IntentFilter intFilter=new IntentFilter(ScanManager.ACTION_SEND_SCAN_RESULT);
		registerReceiver(mResultReceiver, intFilter);
	}

	private void unRegisterReceiver()
	{
		try {
			unregisterReceiver(mResultReceiver);
		} catch (Exception e) {
		}
	}

	@Override
	protected void onPause() {
		super.onPause();
		unRegisterReceiver();
	}

	@Override
	protected void onResume() {
		super.onResume();
		registerReceiver();
	}

	/**
	 * 监听扫码数据的广播，当设置广播输出时作用该方法获取扫码数据
	 */
	private BroadcastReceiver mResultReceiver=new BroadcastReceiver() {

		@Override
		public void onReceive(Context context, Intent intent) {
			String action=intent.getAction();
			if(ScanManager.ACTION_SEND_SCAN_RESULT.equals(action)){
				byte[] bvalue1=intent.getByteArrayExtra(ScanManager.EXTRA_SCAN_RESULT_ONE_BYTES);
				byte[] bvalue2=intent.getByteArrayExtra(ScanManager.EXTRA_SCAN_RESULT_TWO_BYTES);
				String svalue1=null;
				String svalue2=null;
				try {
					if(bvalue1!=null)
						svalue1=new String(bvalue1,"GBK");
					if(bvalue2!=null)
						svalue2=new String(bvalue1,"GBK");
					svalue1=svalue1==null?"":svalue1;
					svalue2=svalue2==null?"":svalue2;
					tv_broadcast_result.setText(svalue1+"\n"+svalue2);
				} catch (Exception e) {
					e.printStackTrace();
					tv_broadcast_result.setText("data encode failed.");
				}

				Random random = new Random();
				tv_broadcast_result.setTextColor(Color.argb(255, random.nextInt(256),
	                    random.nextInt(256), random.nextInt(256)));
			}
		}
	};

}






















